package com.zjugis.nature.gis.dwg;

import com.onespatial.dwglib.Reader;
import com.onespatial.dwglib.objects.*;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 * @Author 陈俊
 * @Date 2023/8/17 9:18
 * @Version 1.0
 */
public class DwgUtils {

    /**
     * 读取dwg文件获取lwPolyline转为wkt字符串格式
     * @param dwgFile
     * @return
     * @throws IOException
     */
    public static String getWkt(File dwgFile) throws IOException {
        Reader reader = new Reader(dwgFile);
        BlockControlObj blockControl = reader.getBlockControlObject();
        BlockHeader ms = (BlockHeader)blockControl.getModelSpace();
        List<LwPolyline> lwPolylines = new ArrayList<>();
        if (ms != null) {
            for (CadObject entity : ms.getOwnedObjects()) {
                if (entity instanceof LwPolyline){
                    lwPolylines.add((LwPolyline) entity);
                }
            }
        }
        if(lwPolylines.isEmpty()){
            return "";
        }else{
            StringBuilder sb = new StringBuilder();
            if(lwPolylines.size()>1){
                sb.append("MULTIPOLYGON (");
            }else{
                sb.append("POLYGON ");
            }
            lwPolylines.forEach(lwPolyline -> {
                String firstPoint = "";
                String endPoint = "";
                sb.append("((");
                List<LwPolyline.VertexOfLwPolyline> points = lwPolyline.points;
                for (int i = 0; i < points.size(); i++) {
                    String x = points.get(i).vertex.x + "";
                    String y = points.get(i).vertex.y + "";
                    String point = " " + x + " " + y + ",";
                    if (i == 0) {
                        firstPoint = point;
                    }
                    if(i == points.size()-1){
                        endPoint = point;
                    }
                    sb.append(point);
                }
                if(!firstPoint.equals(endPoint)){
                    sb.append(firstPoint);
                }
                sb.delete(sb.length() - 1, sb.length());
                sb.append(" )),");
            });
            sb.delete(sb.length() - 1, sb.length());
            if (lwPolylines.size() > 1) {
                sb.append(")");
            }
            return sb.toString();
        }
    }
}
