package com.zjugis.nature.gis.dxf;

import com.zjugis.nature.gis.exception.LayerNotFoundException;
import org.kabeja.dxf.*;
import org.kabeja.parser.ParseException;
import org.kabeja.parser.Parser;
import org.kabeja.parser.ParserBuilder;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.util.Iterator;
import java.util.List;

/**
 * @Author 陈俊
 * @Date 2023/8/15 14:20
 * @Version 1.0
 */
public class DxfUtils {

    /**
     * 读取dxf文件获取lwPolyline转为wkt字符串格式
     * @param dxfFile
     * @return
     * @throws FileNotFoundException
     * @throws ParseException
     */
    public static String getWkt(File dxfFile) throws FileNotFoundException, ParseException, LayerNotFoundException {
        Parser dxfParser = ParserBuilder.createDefaultParser();
        //需要转换的dxf
        dxfParser.parse(new FileInputStream(dxfFile), "UTF-8");
        DXFDocument doc = dxfParser.getDocument();
        Iterator dxfLayerIterator = doc.getDXFLayerIterator();
        if(dxfLayerIterator.hasNext()){
            DXFLayer dXFLayer = (DXFLayer) dxfLayerIterator.next();
            StringBuilder sb = new StringBuilder();
            if(dXFLayer.hasDXFEntities(DXFConstants.ENTITY_TYPE_LWPOLYLINE)) {
                List plines = dXFLayer.getDXFEntities(DXFConstants.ENTITY_TYPE_LWPOLYLINE);
                if (plines.size() > 1) {
                    sb.append("MULTIPOLYGON (");
                } else {
                    sb.append("POLYGON ");
                }
                plines.forEach(pline -> {
                    DXFLWPolyline dxflwPolyline = (DXFLWPolyline) pline;
                    Iterator vertexIterator = dxflwPolyline.getVertexIterator();
                    int i = 0;
                    String firstPoint = "";
                    String endPoint = "";
                    sb.append("((");
                    while (vertexIterator.hasNext()) {
                        DXFVertex dxfVertex = (DXFVertex) vertexIterator.next();
                        String x = dxfVertex.getPoint().getX() + "";
                        String y = dxfVertex.getPoint().getY() + "";
                        String point = " " + x + " " + y + ",";
                        if (i == 0) {
                            firstPoint = point;
                        }
                        if(!vertexIterator.hasNext()){
                            endPoint = point;
                        }
                        sb.append(point);
                        i++;
                    }
                    if(!firstPoint.equals(endPoint)){
                        sb.append(firstPoint);
                    }
                    sb.delete(sb.length() - 1, sb.length());
                    sb.append(" )),");
                });
                sb.delete(sb.length() - 1, sb.length());
                if (plines.size() > 1) {
                    sb.append(")");
                }
                return sb.toString();
            }
        }else{
            throw new LayerNotFoundException("dxf文件中找不到图层");
        }
        return "";
    }
}
