package com.zjugis.nature.gis.shp;

import com.vividsolutions.jts.geom.GeometryFactory;
import com.vividsolutions.jts.geom.MultiPolygon;
import com.vividsolutions.jts.geom.Polygon;
import com.zjugis.nature.gis.exception.BusinessException;
import org.geotools.data.FeatureSource;
import org.geotools.data.shapefile.ShapefileDataStore;
import org.geotools.feature.FeatureCollection;
import org.geotools.feature.FeatureIterator;
import org.opengis.feature.Property;
import org.opengis.feature.simple.SimpleFeature;
import org.opengis.feature.simple.SimpleFeatureType;

import java.io.File;
import java.io.IOException;
import java.nio.charset.Charset;
import java.util.*;

import static com.zjugis.nature.gis.exception.enums.GlobalErrorCodeConstants.CRS_CODE_ERROR;

/**
 * @Author 陈俊
 * @Date 2023/8/3 16:07
 * @Version 1.0
 */
public class ShpUtils {

    public static String parse(File file) throws IOException {
        ShapefileDataStore shpDataStore = new ShapefileDataStore(file.toURI().toURL());
        shpDataStore.setCharset(Charset.forName("UTF-8"));
        String typeName = shpDataStore.getTypeNames()[0];
        FeatureSource<SimpleFeatureType, SimpleFeature> featureSource;
        featureSource = shpDataStore.getFeatureSource(typeName);
        FeatureCollection<SimpleFeatureType, SimpleFeature> result = featureSource.getFeatures();
        FeatureIterator<SimpleFeature> iterator = result.features();

        List<Polygon> list= new ArrayList<>();
        GeometryFactory factory = null;

        while(iterator.hasNext()) {
            SimpleFeature feature = iterator.next();
            String crsCode = feature.getBounds().getCoordinateReferenceSystem().getCoordinateSystem().getName().getCode();
            if(!"GCS_China_Geodetic_Coordinate_System_2000".equals(crsCode)){
                throw new BusinessException(CRS_CODE_ERROR.getMsg());
            }
            Collection<Property> p = feature.getProperties();
            Iterator<Property> it = p.iterator();
            while(it.hasNext()) {
                Property pro = it.next();
                if (pro.getValue() instanceof MultiPolygon) {
                    MultiPolygon var12 = (MultiPolygon)pro.getValue();

                    for (int i = 0; i < var12.getNumGeometries(); i++) {
                        Polygon polygon = (Polygon) var12.getGeometryN(i);
                        list.add(polygon);
                    }
                    factory = var12.getFactory();

                }
            }
        }
        MultiPolygon geometryCollection = factory.createMultiPolygon(list.toArray(new Polygon[0]));
        return geometryCollection.toString();
    }
}
